@@ -9,14 +9,14 @@
 /// This trait identifies algorithms suitable for sorting slices.
 pub trait SortingAlgorithm {
     /// Sorts the given slice.
-    fn sort<T: Ord>(slice: &mut [T]);
+    fn sort<T: Ord>(&self, slice: &mut [T]);
 }
 
 /// <https://github.com/orlp/pdqsort>
 pub struct PDQsort;
 
 impl SortingAlgorithm for PDQsort {
-    fn sort<T: Ord>(slice: &mut [T]) {
+    fn sort<T: Ord>(&self, slice: &mut [T]) {
         slice.sort_unstable();
     }
 }
@@ -25,14 +25,14 @@ impl SortingAlgorithm for PDQsort {
 pub struct Timsort;
 
 impl SortingAlgorithm for Timsort {
-    fn sort<T: Ord>(slice: &mut [T]) {
+    fn sort<T: Ord>(&self, slice: &mut [T]) {
         slice.sort();
     }
 }
 
-/// Sorts the given slice using the [`SortingAlgorithm`] `SA`.
-pub fn sort<T: Ord, SA: SortingAlgorithm>(slice: &mut [T]) {
-    SA::sort(slice);
+/// Sorts the given slice using a specific [`SortingAlgorithm`].
+pub fn sort<T: Ord, A: SortingAlgorithm>(slice: &mut [T], algo: &A) {
+    algo.sort(slice);
 }
 
 /// Checks if the elements of the slice are sorted.
@@ -58,21 +58,21 @@ mod tests {
     #[test]
     fn timsort() {
         let mut v = [1, 2, 3, 6, 3, 1, 3, -4, 6, -2, 3, 0, 5, 3, 0, 12];
-        Timsort::sort(&mut v);
+        Timsort.sort(&mut v);
         assert!(is_sorted(&v));
     }
 
     #[test]
     fn pdqsort() {
         let mut v = [1, 2, 3, 6, 3, 1, 3, -4, 6, -2, 3, 0, 5, 3, 0, 12];
-        PDQsort::sort(&mut v);
+        PDQsort.sort(&mut v);
         assert!(is_sorted(&v));
     }
 
     #[test]
     fn it_sorts() {
         let mut v = [1, 2, 3, 6, 3, 1, 3, -4, 6, -2, 3, 0, 5, 3, 0, 12];
-        sort::<_, Timsort>(&mut v);
+        sort(&mut v, &Timsort);
         assert!(is_sorted(&v));
     }
 }
